@@ -164,9 +164,46 @@ impl Eq for LR1State {}
 pub fn printstate(state:&LR1State,Gmr:&Grammar) 
 {
   println!("state {}:",state.index);
+  let mut lamap:HashMap<(usize,usize),Vec<&String>> = HashMap::with_capacity(Gmr.Rules.len()*4);
+  for item in &state.items
+  {
+     let laset:&mut Vec<&String> = match lamap.get_mut(&(item.ri,item.pi)) {
+        Some(x) => x,
+        None => {
+           let mut newset = Vec::<&String>::with_capacity(16);
+           lamap.insert((item.ri,item.pi),newset);
+           lamap.get_mut(&(item.ri,item.pi)).unwrap()
+        },
+     };//match
+     laset.push(&item.la);
+  }
+  for (ri,pi) in lamap.keys()
+  {
+    let ref lhs_sym = Gmr.Rules[*ri].lhs.sym;
+     let ref rhs = Gmr.Rules[*ri].rhs;
+     print!("  ({}) {} --> ",ri,lhs_sym);
+     let mut position = 0;
+     for gsym in rhs 
+     {
+       if &position==pi {print!(".");}
+       print!("{} ",gsym.sym);
+       position+=1;
+     }
+     if &position==pi {print!(". ");}
+     print!(" {{ ");
+     for la in lamap.get(&(*ri,*pi)).unwrap()
+     {
+       print!("{},",la);
+     }
+     println!(" }}");
+  }//for key
+}//printstate
+pub fn printstate_raw(state:&LR1State,Gmr:&Grammar) 
+{
   for item in &state.items
   { printitem(item,Gmr); }
-}//printstate
+}
+
 
 
 pub fn stateclosure(mut state:LR1State, Gmr:&Grammar)
@@ -220,13 +257,18 @@ pub fn stateclosure(mut state:LR1State, Gmr:&Grammar)
 
 /// this enum is only exported because it's used by the generated parsers.
 /// There is no reason to use it in other programs.
-#[derive(Clone,PartialEq,Eq,Debug)]
+#[derive(Copy,Clone,PartialEq,Eq,Debug)]
 pub enum Stateaction {
   Shift(usize),     // shift then go to state index
   Reduce(usize),    // reduce by rule index
   Gotonext(usize),  // folded into same table, only for non-terminals
   Accept,
-  Error(String),
+  /// note: this has been changed after version 0.1.1 from String to
+  /// &'static str for increased efficiency. Error action entries are
+  /// not generated by rustlr: they can only be added with the parser's
+  /// training capability.  Parsers already trained can be hand-modified
+  /// by removing all instances of ".to_string()" from the load_extras function.
+  Error(&'static str),
 }
 
 // abstract parser struct
@@ -297,7 +339,7 @@ impl Statemachine
        }
      }// lalr or lr1
 
-     if TRACE==2 {println!("transition to state {} from state {}, symbol {}..",toadd,psi,&nextsym);}
+     if TRACE>3 {println!("transition to state {} from state {}, symbol {}..",toadd,psi,&nextsym);}
      if toadd==newstateindex {  // add new state
        if TRACE>2 {printstate(&state,&self.Gmr);}
        indices.insert(newstateindex); // add to StateLookup index hashset
@@ -313,6 +355,7 @@ impl Statemachine
      let currentaction = self.FSM[psi].get(&nextsym);
      let mut changefsm = true;
      match currentaction {   // detect shift-reduce conflict
+       Some(Accept) => { changefsm=false; },
        Some(Reduce(ri2)) =>  {
          let prec2 = self.Gmr.Rules[*ri2].precedence;
          let prec1 = gsymbol.precedence;
@@ -320,7 +363,6 @@ impl Statemachine
          else if prec2.abs()>prec1.abs() {changefsm=false;} // still reduce
          if TRACE>0 {println!("shift-reduce conflict resolved by operator precedence/associativity:"); printrulela(*ri2,&self.Gmr,&nextsym); /*printstate(&self.States[psi],&self.Gmr);*/}
        },
-       Some(Accept) => {changefsm=false;},
        _ => {},
      }// match for conflict detection
      if changefsm {self.FSM[psi].insert(nextsym,newaction);}
@@ -339,7 +381,7 @@ impl Statemachine
               // determine if this is a reduce item
               if item.pi >= Gmr.Rules[item.ri].rhs.len() {
                  if TRACE>1 {print!("LALR MERGE: ");}
-                 Statemachine::addreduce(FSM,Gmr,item,si);
+                 Statemachine::aditdreduce(FSM,Gmr,item,si);
               }
               States[si].items.insert(item.clone());
           }// new item needs to be inserted
@@ -353,12 +395,17 @@ impl Statemachine
   // it assumes that the . is at the right end of the rule
   fn addreduce(FSM: &mut Vec<HashMap<String,Stateaction>>, Gmr:&Grammar, item:&LRitem, si:usize)
   {
+     let isaccept = (item.ri == Gmr.Rules.len()-1 && item.la=="EOF");
      let currentaction = FSM[si].get(&item.la);
      let mut changefsm = true;
      let ri1 = &item.ri;
      /// detect CONFLICT HERE
      match currentaction {
-        Some(Reduce(ri2)) if ri2<ri1 => {
+        Some(Accept) => {
+          changefsm = false;
+          if !isaccept {println!("Reduce({})-Accept conflict resolved in favor of Accept",ri1)}
+        },
+        Some(Reduce(ri2)) if ri2<ri1 && !isaccept => {
            changefsm=false;
            println!("Reduce-Reduce Conflict conflicted detected between rules {} and {}, resolved in favor of {}",ri2,ri1,ri2);
            printrulela(*ri1,Gmr,&item.la);  printrulela(*ri2,Gmr,&item.la);
@@ -369,9 +416,8 @@ impl Statemachine
            printrulela(*ri1,Gmr,&item.la);  printrulela(*ri2,Gmr,&item.la); 
            //printstate(&self.States[si],Gmr);            
         },
-        Some(Reduce(ri2)) if ri2==ri1 => {changefsm=false;},
-        Some(Accept) => {changefsm = false;},
-        Some(Shift(_)) => {   // shift-reduce conflict
+        Some(Reduce(ri2)) if ri2==ri1 && !isaccept => {changefsm=false;},
+        Some(Shift(_)) if !isaccept => {   // shift-reduce conflict
            let prec1 = Gmr.Rules[item.ri].precedence;
            let prec2 = Gmr.Symbols[*Gmr.Symhash.get(&item.la).unwrap()].precedence;
 
@@ -381,9 +427,12 @@ impl Statemachine
         },
        _ => {},
      }//match to detect conflict
+     // special case: current action should be Accept:
      if changefsm {   // only Reduce/Accept added here
         // accept or reduce
-        if item.ri==Gmr.Rules.len()-1 && item.la=="EOF"  { // start rule
+        if isaccept /*item.ri==Gmr.Rules.len()-1 && item.la=="EOF"*/  {
+           if let None = &currentaction {}
+           else {println!("Accept has precedence over {:?}",&currentaction);}
            FSM[si].insert(item.la.clone(),Stateaction::Accept);
         }
         else {
@@ -424,43 +473,6 @@ impl Statemachine
        else // . at end of production, this is a reduce situation
        {
           Statemachine::addreduce(&mut self.FSM,&self.Gmr,item,si);
-           /*
-          let currentaction = self.FSM[si].get(&item.la);
-          let mut changefsm = true;
-          let ri1 = &item.ri;
-          /// detect CONFLICT HERE
-          match currentaction {
-            Some(Reduce(ri2)) if ri2<ri1 => {
-              changefsm=false;
-              println!("Reduce-Reduce Conflict conflicted detected between rules {} and {}, resolved in favor of {}",ri2,ri1,ri2);
-              printstate(&self.States[si],&self.Gmr);
-            },
-            Some(Reduce(ri2)) if ri2>ri1 => {
-              println!("Reduce-Reduce Conflict conflicted detected between rules {} and {}, resolved in favor of {}",ri2,ri1,ri1);
-              printstate(&self.States[si],&self.Gmr);            
-            },
-            Some(Accept) => {changefsm = false;},
-            Some(Shift(_)) => {   // shift-reduce conflict
-              let prec1 = self.Gmr.Rules[item.ri].precedence;
-              let prec2 = self.Gmr.Symbols[*self.Gmr.Symhash.get(&item.la).unwrap()].precedence;
-              //let prec2 = self.Gmr.Symbols.get(&item.la).unwrap().precedence;
-              if prec1==prec2 && prec1<0 {changefsm=false;} // assume right-associative
-              else if prec2.abs()>prec1.abs() {changefsm=false;} // still shift 
-              if TRACE>4 {println!("shift-reduce conflict resolved by operator precedence/associativity:"); printstate(&self.States[si],&self.Gmr);}
-            },
-            _ => {},
-          }//match to detect conflict
-
-          if changefsm {   // only Reduce/Accept added here
-             // accept or reduce
-             if item.ri==self.Gmr.Rules.len()-1 && item.la=="EOF"  { // start rule
-               self.FSM[si].insert(item.la.clone(),Stateaction::Accept);
-             }
-             else {
-               self.FSM[si].insert(item.la.clone(),Stateaction::Reduce(item.ri));
-             }
-          }// add reduce action
-          */
        } // set reduce action
      }// for each item 
      // form closures for all new states and add to self.States list
@@ -522,8 +534,8 @@ pub fn decode_action(code:u64) -> Stateaction
       (1,si) => Gotonext(si as usize),
       (2,ri) => Reduce(ri as usize),
       (3,_)  => Accept,
-      (4,x)  => Error(x.to_string()),
-      _      => Error("unrecognized action in TABLE".to_owned()),
+      (4,x)  => Error("shouldn't be here"),
+      _      => Error("unrecognized action in TABLE"),
     }
 }//decode - must be independent function seen by parsers
 
