@@ -18,6 +18,7 @@ use std::io::prelude::*;
 use std::path::Path;
 use std::mem;
 use crate::{TRACE,Lexer,Lextoken,Stateaction,Statemachine,augment_file};
+use crate::{LBox};
 use crate::Stateaction::*;
 
 /// this structure is only exported because it is required by the generated parsers.
@@ -34,7 +35,7 @@ impl<AT:Default,ET:Default> RProduction<AT,ET>
   {
      RProduction {
        lhs : lh,
-       Ruleaction : |p|{AT::default()},
+       Ruleaction : |p|{<AT>::default()},
      }
   }
 }//impl RProduction
@@ -77,7 +78,7 @@ pub struct RuntimeParser<AT:Default,ET:Default>
 //  pub recover : HashSet<&'static str>, // for error recovery
   pub resynch : HashSet<&'static str>,
   pub Errsym : &'static str,
-  err_occured : bool,
+  err_occurred : bool,
   pub linenum : usize,
   pub column : usize,
   report_line : usize,
@@ -102,7 +103,7 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
          exstate : ET::default(),
          stack : Vec::with_capacity(1024),
          Errsym : "",
-         err_occured : false,
+         err_occurred : false,
          linenum : 0,
          column : 0,
          report_line : 0,
@@ -124,12 +125,13 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
     /// "reduce" action of the parser.
     pub fn abort(&mut self, msg:&str)
     {
-       println!("\n!!!Parsing Aborted: {}",msg);
-       self.err_occured = true;
+       eprintln!("\n!!!Parsing Aborted: {}",msg);
+       self.err_occurred = true;
        self.stopparsing=true;
     }
 
-    /// may be called from grammar semantic actions to report error
+    /// may be called from grammar semantic actions to report error.
+    /// this report function will print to stdout. 
     pub fn report(&mut self, errmsg:&str)  
     {      // linenum must be set prior to call
        if (self.report_line != self.linenum || self.linenum==0)  {
@@ -140,7 +142,7 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
        else {
          print!(" {} ",errmsg);
        }
-       self.err_occured = true;
+       self.err_occurred = true;
     }
 
     //called to simulate a shift
@@ -167,14 +169,14 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
                 // DO NOT CHANGE LOOKAHEAD AFTER REDUCE!
               }// goto next state after reduce
               else {
-                self.report("no suitable action can be taken");
+                self.report("state transition table corrupted: no suitable action after reduce");
                 self.stopparsing=true;
               }
     }//reduce
 
     /// can be called to determine if an error occurred during parsing.  The parser
     /// will not panic.
-    pub fn error_occurred(&self) -> bool {self.err_occured}
+    pub fn error_occurred(&self) -> bool {self.err_occurred}
 
     fn nexttoken(&self, tokenizer:&mut dyn Lexer<AT>) -> Lextoken<AT>
     {
@@ -182,20 +184,19 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
         else { Lextoken{sym:"EOF".to_owned(),  value:AT::default()} } 
     }
     // parse does not reset state stack
-    
-    /// this function is used to invoke the generated parser returned by
-    /// the generated parser program's make_parser function.
-    pub fn parse(&mut self, tokenizer:&mut dyn Lexer<AT>) -> AT
+
+    //original
+    fn parse0(&mut self, tokenizer:&mut dyn Lexer<AT>) -> AT
     {
-       self.err_occured = false;
+       self.err_occurred = false;
        self.stack.clear();
        let mut eofcount = 0;
 //       self.exstate = ET::default(); ???
        let mut result = AT::default();
        // push state 0 on stack:
        self.stack.push(Stackelement {si:0, value:AT::default()});
-       let unexpected = Stateaction::Error(String::from("unexpected end of input"));
-       let mut action = unexpected; //Stateaction::Error(String::from("get started"));
+       let unexpected = Stateaction::Error("unexpected end of input");
+       let mut action = unexpected; 
        self.stopparsing = false;
        let mut lookahead = Lextoken{sym:"EOF".to_owned(),value:AT::default()}; 
        if let Some(tok) = tokenizer.nextsym() {lookahead=tok;}
@@ -226,7 +227,7 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
             }// lookahead is not a grammar sym
 
             let errmsg = if let Some(Error(em)) = &actionopt {
-               format!("unexpected symbol {}, **{}** ..",lksym,em)
+               format!("unexpected symbol {}, ** {} ** ..",lksym,em)
             } else {format!("unexpected symbol {} ..",lksym)};
 
             self.report(&errmsg);
@@ -238,7 +239,7 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
               print!("\n>>>TRAINER: is this error message adequate? If not, enter a better one: ");
               let rrrflush = io::stdout().flush();
               if let Ok(n) = io::stdin().read_line(&mut inp) {
-                if inp.len()>4 && self.Symset.contains(lksym) /*&& !self.trained.contains_key(&(cstate,csym.clone()))*/ {
+                if inp.len()>5 && self.Symset.contains(lksym) /*&& !self.trained.contains_key(&(cstate,csym.clone()))*/ {
                   print!(">>>TRAINER: should this message be given for all unexpected symbols in the current state? (default yes) ");
                   let rrrflush2 = io::stdout().flush();
                   let mut inp2 = String::new();
@@ -251,7 +252,7 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
                      }
                   }// read ok
                 }// unexpected symbol is grammar sym
-                else if inp.len()>4 && !self.Symset.contains(lksym) /*&& !self.trained.contains_key(&(cstate,String::from("ANY_ERROR")))*/ {
+                else if inp.len()>5 && !self.Symset.contains(lksym) /*&& !self.trained.contains_key(&(cstate,String::from("ANY_ERROR")))*/ {
                   self.trained.insert((cstate,String::from("ANY_ERROR")),inp);
                 }
                 
@@ -426,20 +427,22 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
           //panic!("!!!Parsing failed on line {}, next symbol {}: {}",tokenizer.linenum(),&lookahead.sym,msg);
           self.report(&format!("failure with next symbol {}",tokenizer.linenum()));
        }
-       //if self.err_occured {result = AT::default(); }
+       //if self.err_occurred {result = AT::default(); }
        return result;
-    }//parse
+    }//parse0
 
     /// Parse in training mode: when an error occurs, the parser will
     /// ask the human trainer for an appropriate error message: it will
     /// then insert an entry into its state transition table to
     /// give the same error message on future errors of the same type.
-    /// If the error is caused by an expected token that is recognized
+    /// If the error is caused by an unexpected token that is recognized
     /// as a terminal symbol of the grammar, the trainer can select to
     /// enter the entry 
     /// under the reserved ANY_ERROR symbol. If the unexpected token is
     /// not recognized as a grammar symbol, then the entry will always
-    /// be entered under ANY_ERROR.
+    /// be entered under ANY_ERROR.  ANY_ERROR entries for a state will match
+    /// all future unexpected symbols for that state: however, entries for
+    /// valid grammar symbols will still override the generic entry.
     ///
     /// Example: with the parser for this [toy grammar](https://cs.hofstra.edu/~cscccl/rustlr_project/cpm.grammar), parse_train can run as follows:
     ///```ignore
@@ -450,26 +453,37 @@ impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
     ///```
     /// (ignore the column number as the lexer for this toy language does not implement it)
     ///
-    /// parse_train will then save a [modified version](https://cs.hofstra.edu/~cscccl/rustlr_project/augmented_cpmparser.rs) of the parser as specified
-    /// by the filename argument.  When the augmented parser is used, it will
+    /// parse_train will then modify [the parser file](https://cs.hofstra.edu/~cscccl/rustlr_project/augmented_cpmparser.rs) as specified
+    /// by the filename (path) argument.  When the augmented parser is used, it will
     /// give a more helpful error message:
     ///```
     /// Write something in C+- : cout << x endl
     /// ERROR on line 1, column 0: unexpected symbol endl, **need another <<** ..
     ///```
+    ///
+    /// parse_train calls parse, which uses stdin/stdout for user interface.
     pub fn parse_train(&mut self, tokenizer:&mut dyn Lexer<AT>, filename:&str) -> AT
     {
       self.training = true;
       let result = self.parse(tokenizer);
       if let Err(m) = augment_file(filename,self) {
-        println!("Error in augmenting parser: {:?}",m)
+        eprintln!("Error in augmenting parser: {:?}",m)
       }
+      self.training = false;
       return result;
-    }
+    }//parse_train
+
+    /// creates a [LBox] smart pointer that includes line/column/src information;
+    /// should be called from the semantic actions of a grammar rule, e.g.
+    ///```ignore
+    ///   E --> E:a + E:b {PlusExpr(parser.lb(a),parser.lb(b))}
+    ///```
+    pub fn lb(&self,e:AT) -> LBox<AT> { LBox::new(e,self.linenum,self.column) }
 }// impl RuntimeParser
 
 
-////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////
 //// new version of write_fsm:
 
 impl Statemachine
@@ -483,6 +497,7 @@ impl Statemachine
 #![allow(non_camel_case_types)]
 #![allow(unused_parens)]
 #![allow(unused_mut)]
+#![allow(unused_imports)]
 #![allow(unused_assignments)]
 extern crate rustlr;
 use rustlr::{{RuntimeParser,RProduction,Stateaction,decode_action}};\n")?;
@@ -553,7 +568,7 @@ use rustlr::{{RuntimeParser,RProduction,Stateaction,decode_action}};\n")?;
       //if semaction.len()<1 {semaction = "}}";}
       //if al>1 {semaction = semaction.substring(0,al-1);}
       if semaction.len()>1 {write!(fd,"{};\n",semaction.trim_end())?;}
-      else {write!(fd," return {}::default();}};\n",absyn)?;}
+      else {write!(fd," return <{}>::default();}};\n",absyn)?;}
       write!(fd," parser1.Rules.push(rule);\n")?;
     }// for each rule
     write!(fd," parser1.Errsym = \"{}\";\n",&self.Gmr.Errsym)?;
@@ -575,7 +590,7 @@ use rustlr::{{RuntimeParser,RProduction,Stateaction,decode_action}};\n")?;
 
     ////// Augment!
     write!(fd,"fn load_extras(parser:&mut RuntimeParser<{},{}>)\n{{\n",absyn,extype)?;
-    write!(fd,"}}//end of load_extras: don't change this line as it affects augmentation")?;
+    write!(fd,"}}//end of load_extras: don't change this line as it affects augmentation\n")?;
     Ok(())
   }//writeparser
 
@@ -625,7 +640,7 @@ use rustlr::{{RuntimeParser,RProduction,Stateaction}};\n")?;
       //if semaction.len()<1 {semaction = "}}";}
       //if al>1 {semaction = semaction.substring(0,al-1);}
       if semaction.len()>1 {write!(fd,"{};\n",semaction.trim_end())?;}
-      else {write!(fd," return {}::default();}};\n",absyn)?;}
+      else {write!(fd," return <{}>::default();}};\n",absyn)?;}
       write!(fd," parser1.Rules.push(rule);\n")?;
     }// for each rule
     write!(fd," parser1.Errsym = \"{}\";\n",&self.Gmr.Errsym)?;
@@ -649,7 +664,7 @@ use rustlr::{{RuntimeParser,RProduction,Stateaction}};\n")?;
 } // impl Statemachine
 
 
-
+//// independent function
     fn iserror(actionopt:&Option<&Stateaction>) -> bool
     {
        match actionopt {
@@ -659,3 +674,253 @@ use rustlr::{{RuntimeParser,RProduction,Stateaction}};\n")?;
          }
     }//iserror
 
+
+
+
+
+///////////////////////////////////////////////////////////////////////////
+////// reimplementing the parsing algorithm more modularly, with aim of
+////// allowing custom parsers
+//////////// errors should compile a report
+
+/// In case one wishes to construct a parser error-reporting interface
+/// that's different from the supplied [RuntimeParser::parse] function,
+/// which prints to stdout, a function of ErrorReporter type can be defined
+/// and used in conjuction with [RuntimeParser::parse_core].
+pub type ErrorReporter<AT,ET> =
+  fn(&mut RuntimeParser<AT,ET>, &Lextoken<AT>, &Option<Stateaction>);
+  
+
+impl<AT:Default,ET:Default> RuntimeParser<AT,ET>
+{
+
+  // reduce already implemented
+  // no separate function for gotonext - part of reduce
+
+  /// this is the LR parser shift action: push the next state, along with the
+  /// value of the current lookahead token onto the parse stack, returns the
+  /// next token
+  fn shift(&mut self, nextstate:usize, lookahead:Lextoken<AT>, tokenizer:&mut dyn Lexer<AT>) -> Lextoken<AT>
+  {
+     self.stack.push(Stackelement{si:nextstate,value:lookahead.value});
+     self.nexttoken(tokenizer)
+  }
+
+  /// This is the core parser, which expects a ErrorReporter function to be
+  /// passed in as an argument.
+  pub fn parse_core(&mut self, tokenizer:&mut dyn Lexer<AT>, err_reporter:ErrorReporter<AT,ET>) -> AT
+  {
+    self.stack.clear();
+    self.err_occurred = false;
+    let mut result = AT::default();
+    self.stack.push(Stackelement {si:0, value:AT::default()});
+    self.stopparsing = false;
+    let mut action = Stateaction::Error("");
+    let mut lookahead = Lextoken{sym:"EOF".to_owned(),value:AT::default()};
+    if let Some(tok) = tokenizer.nextsym() {lookahead=tok;}
+    else {self.stopparsing=true;}
+
+    while !self.stopparsing
+    {
+      self.linenum = tokenizer.linenum(); self.column=tokenizer.column();
+      let currentstate = self.stack[self.stack.len()-1].si;
+      let mut actionopt = self.RSM[currentstate].get(lookahead.sym.as_str());
+      let actclone:Option<Stateaction> = match actionopt {
+        Some(a) => Some(*a),
+        None => None,
+      };
+      if iserror(&actionopt) {  // either None or Error
+        if !self.err_occurred {self.err_occurred = true;}
+        err_reporter(self,&lookahead,&actclone);
+        match self.error_recover(&mut lookahead,tokenizer) {
+          None => { self.stopparsing=true; break; }
+          Some(act) => {action = act;},
+        }//match
+      }// iserror
+      else { action = actclone.unwrap(); }
+      match &action {
+        Shift(nextstate) => {
+           lookahead = self.shift(*nextstate,lookahead,tokenizer);
+        },
+        Reduce(rulei) => { self.reduce(rulei); },
+        Accept => {
+          self.stopparsing=true;
+          if self.stack.len()>0 {result = self.stack.pop().unwrap().value;}
+          else {self.err_occurred=true;}
+        },
+        _ => {}, // continue
+      }//match action
+    }// main parse loop
+    return result;
+  }//parse_core
+
+  /// this function is used to invoke the generated parser returned by
+  /// the generated parser program's make_parser function.  This
+  /// function invokes parse_core with err_report_train as the ErrorReporter
+  /// function.
+  pub fn parse(&mut self, tokenizer:&mut dyn Lexer<AT>) -> AT
+  {
+     self.parse_core(tokenizer,err_report_train)
+  }
+
+
+  /// Error recovery routine of rustlr, separate from error_reporter.
+  /// This function will modify the parser and lookahead symbol and return
+  /// either the next action the parser should take (if recovery succeeded)
+  /// or None if recovery failed.
+  pub fn error_recover<'t>(&mut self, lookahead:&mut Lextoken<AT>,tokenizer:&mut dyn Lexer<AT>) -> Option<Stateaction>
+  {
+    let mut erraction = None;
+    ///// prefer to ue Errsym method
+    if self.Errsym.len()>0 {
+      let errsym = self.Errsym;
+      // lookdown stack for state with trainsiton on Errsym
+      // but that could be current state too (start at top)
+      let mut k = self.stack.len(); // offset by 1 because of usize
+      let mut spos = k+1;
+      while k>0 && spos>k
+      {
+        let ksi = self.stack[k-1].si;
+        erraction = self.RSM[ksi].get(errsym);
+        if let None = erraction {k-=1;} else {spos=k;}
+      }//while k>0
+      if spos==k { self.stack.truncate(k); } // new current state revealed
+      // run all reduce actions that are valid before the Errsym:
+      while let Some(Reduce(ri)) = erraction // keep reducing
+      {
+       //self.reduce(ri); // borrow error- only need mut self.stack
+              let rulei = &self.Rules[*ri];
+              let ruleilhs = rulei.lhs; // &'static : Copy
+              let val = (rulei.Ruleaction)(self); // calls delegate function
+              let newtop = self.stack[self.stack.len()-1].si; 
+              let gotonopt = self.RSM[newtop].get(ruleilhs);
+              match gotonopt {
+                Some(Gotonext(nsi)) => { 
+                  self.stack.push(Stackelement{si:*nsi,value:val});
+                },// goto next state after reduce
+                _ => {self.abort("recovery failed"); },
+              }//match
+              // end reduce
+              let tos=self.stack[self.stack.len()-1].si;
+              erraction = self.RSM[tos].get(self.Errsym);
+      } // while let erraction is reduce
+      // remaining defined action on Errsym must be shift
+      if let Some(Shift(i)) = erraction { // simulate shift errsym 
+          self.stack.push(Stackelement{si:*i,value:AT::default()});
+          // keep lookahead until action is found that transitions from
+          // current state (i). but skipping ahead without reducing
+          // the error production is not a good idea
+          while let None = self.RSM[*i].get(&lookahead.sym[..]) {
+            if &lookahead.sym[..]=="EOF" {break;}
+            *lookahead = self.nexttoken(tokenizer);
+          }//while let
+          // either at end of input or found action on next symbol
+          erraction = self.RSM[*i].get(&lookahead.sym[..]);
+      } // if shift action found down under stack
+    }//errsym exists
+
+    // at this point, if erraction is None, then Errsym failed to recover,
+    // try the resynch symbol method next ...
+    if iserror(&erraction) && self.resynch.len()>0 {
+      while &lookahead.sym!="EOF" &&
+        !self.resynch.contains(&lookahead.sym[..]) {
+        self.linenum = tokenizer.linenum(); self.column = tokenizer.column();
+        *lookahead = self.nexttoken(tokenizer);
+      }//while
+      if &lookahead.sym!="EOF" {
+        // look for state on stack that has action defined on next symbol
+        self.linenum = tokenizer.linenum(); self.column = tokenizer.column();    
+        *lookahead = self.nexttoken(tokenizer); // skipp err-causing symbol
+      }
+      let mut k = self.stack.len()-1; // offset by 1 because of usize
+      let mut position = 0;
+      while k>0 && erraction==None
+      {
+         let ksi = self.stack[k-1].si;
+         erraction = self.RSM[ksi].get(&lookahead.sym[..]);
+         if let None=erraction {k-=1;}
+      }//while k>0 && erraction==None
+      match erraction {
+        None => {}, // do nothing, whill shift next symbol
+        _ => { self.stack.truncate(k);},//pop stack
+      }//match
+   }// there are resync symbols
+
+   // at this point, if erraction is None, then resynch recovery failed too.
+   // only action left is to skip ahead...
+   let mut eofcx = 0;
+   while iserror(&erraction) && eofcx<1 { //skip input
+      self.linenum = tokenizer.linenum(); self.column = tokenizer.column();
+      *lookahead = self.nexttoken(tokenizer);
+      if &lookahead.sym=="EOF" {eofcx+=1;}
+      let csi =self.stack[self.stack.len()-1].si;
+      erraction = self.RSM[csi].get(&lookahead.sym[..]);
+   }// skip ahead
+   match erraction {
+     None => None,
+     Some(act) => Some(*act),
+   }//return match
+  }//error_recover
+
+}//imple RuntimeParser 2
+
+
+/// default ErrorReporter, with training ability
+pub fn err_report_train<AT:Default,ET:Default>(parser:&mut RuntimeParser<AT,ET>, lookahead:&Lextoken<AT>, mut erropt:&Option<Stateaction>)
+{
+  // known that actionop is None or Some(Error(_))
+  let cstate = parser.stack[parser.stack.len()-1].si;
+  let mut actionopt = if let Some(act)=erropt {Some(act)} else {None};
+  let lksym = &lookahead.sym[..];
+  // is lookahead recognized as a grammar symbol?
+  // if actionopt is NONE, check entry for ANY_ERROR            
+  if parser.Symset.contains(lksym) {
+     if let None=actionopt {
+        actionopt = parser.RSM[cstate].get("ANY_ERROR");
+     }
+  }// lookahead is recognized grammar sym
+  else {
+     actionopt = parser.RSM[cstate].get("ANY_ERROR");
+  }// lookahead is not a grammar sym
+  let errmsg = if let Some(Error(em)) = &actionopt {
+    format!("unexpected symbol {}, ** {} ** ..",lksym,em)
+  } else {format!("unexpected symbol {} .. ",lksym)};
+
+  parser.report(&errmsg);
+         
+  if parser.training {  /////// TRAINING MODE:
+    let cstate = parser.stack[parser.stack.len()-1].si;
+    let csym = lookahead.sym.clone();
+    let mut inp = String::from("");
+    print!("\n>>>TRAINER: if this message is not adequate (for state {}), enter a replacement (default no change): ",cstate);
+    let rrrflush = io::stdout().flush();
+    if let Ok(n) = io::stdin().read_line(&mut inp) {
+       if inp.len()>5 && parser.Symset.contains(lksym) {
+         print!(">>>TRAINER: should this message be given for all unexpected symbols in the current state? (default yes) ");
+        let rrrflush2 = io::stdout().flush();
+        let mut inp2 = String::new();
+        if let Ok(n) = io::stdin().read_line(&mut inp2) {
+            if inp2.trim()=="no" || inp2.trim()=="No" {
+               parser.trained.insert((cstate,csym),inp);
+            }
+            else  {// insert for any error
+                       parser.trained.insert((cstate,String::from("ANY_ERROR")),inp);
+            }
+        }// read ok
+       }// unexpected symbol is grammar sym
+       else if inp.len()>5 && !parser.Symset.contains(lksym) {
+         parser.trained.insert((cstate,String::from("ANY_ERROR")),inp);
+       }
+    }// process user response
+  }//if training   //// END TRAINING MODE
+
+}// default errorreporter
+
+
+/////// training from file and script
+struct trainscript<AT:Default,ET:Default>
+{
+  parser: RuntimeParser<AT,ET>,
+  srcfile: String,
+  errscript: String,
+}
